package com.arunta.app.remote.example;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.rmi.RemoteException;
import java.util.ArrayList;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.LogManager;
import org.apache.log4j.SimpleLayout;

import com.arunta.app.bom.Recording;
import com.arunta.app.bom.SecurityKey;
import com.arunta.app.remote.TRDatabaseServiceRI;
import com.arunta.app.remote.TRPlayServiceRI;
import com.arunta.app.remote.TRRemoteUtility;
import com.arunta.app.remote.TRSystemServiceRI;
import com.arunta.base.audio.streaming.StreamPacketReceiver;
import com.arunta.base.exception.DbBusyException;
import com.arunta.base.exception.DbUnavailableException;
import com.arunta.base.exception.SecurityException;
import com.arunta.base.exception.TooManySessionsException;
import com.arunta.base.search.SearchCriteria;

public class TRPlaybackExample {

    public static void main(String[] args) {
        
        System.setProperty("log4j.defaultInitOverride","true");
        LogManager.resetConfiguration();
        LogManager.getRootLogger().addAppender(new ConsoleAppender(new SimpleLayout(), "System.out"));
        
        // --
        // Connect and get a security key
        // --
        
        // From the Remote Manager configuration ...
        final String rmIpAddress = "192.168.4.115";
        final int rmPort = 10010;
        final String trPassword = "0000";

        TRSystemServiceRI trSystemService = null;
        try {
            trSystemService = TRRemoteUtility.getSystemService(rmIpAddress, rmPort);;
        } 
        catch (RemoteException e) {
            e.printStackTrace();
            System.exit(1);
        } 

        SecurityKey securityKey = null;
        try {
            securityKey = trSystemService.logon(trPassword, "example");
        } 
        catch (RemoteException e) {
            e.printStackTrace();
            System.exit(1);
        } 
        catch (SecurityException e) {
            e.printStackTrace();
            System.exit(1);
        }

        // --
        // Obtain reference to the remote interface for the playback service
        // and database service
        // --
        
        TRPlayServiceRI trPlayService = null;
        try {
            trPlayService = TRRemoteUtility.getPlayService(rmIpAddress, rmPort);
        }
        catch (RemoteException e) {
            e.printStackTrace();
            System.exit(1);
        } 
        
        TRDatabaseServiceRI trDatabaseService = null;
        try {
            trDatabaseService = TRRemoteUtility.getDatabaseService(rmIpAddress, rmPort);
        }
        catch (RemoteException e) {
            e.printStackTrace();
            System.exit(1);
        } 
        
        // --
        // Start a Playback Packet Receiver
        // --
        
        final String localIpAddress = "192.168.4.212";
        final int localPort = 10099;

        InetSocketAddress socketAddress = new InetSocketAddress(localIpAddress, localPort);
        
        StreamPacketReceiver mpr = new StreamPacketReceiver(socketAddress);
        mpr.setPacketListener(new TRMonitoringClient());
        try {
            mpr.startProcessing();
        } 
        catch (IOException e) {
            e.printStackTrace();
            System.exit(1);
        }

        // --
        // Find recorings and play the most recent one
        // --
        SearchCriteria searchCriteria = new SearchCriteria(10);
        
        ArrayList recordingList = null;
        try {
            recordingList = 
                trDatabaseService.getMatchingRecordings(securityKey.toString(), searchCriteria);
        } 
        catch (RemoteException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        } 
        catch (DbBusyException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        }
        catch (DbUnavailableException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        } 
        catch (SecurityException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        }

        if ((recordingList == null) || recordingList.isEmpty()) {
            mpr.stopProcessing();
            System.out.println("No recordings to play.");
            System.exit(0);
        }
        
        System.out.println("Recordings to play:");
        for (int idx = 0; idx < recordingList.size(); idx++) {
            Recording r = (Recording)recordingList.get(idx);
            System.out.println(r.getFileName());
        }

        Recording recordingToPlay = (Recording)recordingList.get(0);

        String sessionKey = null;
        try {
            sessionKey = 
                trPlayService.startRemotePlayback(securityKey.toString(), socketAddress, new String[] { recordingToPlay.getFileName() });
            
            trPlayService.play(securityKey.toString(), sessionKey);
        } 
        catch (RemoteException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        } 
        catch (IOException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        } 
        catch (TooManySessionsException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        } 
        catch (SecurityException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        }
        
        
        // --
        // At this point the TR should be sending packets to this client.
        // --
        
        // --
        // Ctrl-C stops the show.
        // --
        final Object end = new Object();
        Runtime.getRuntime().addShutdownHook(new Thread() {
           public void run() {
               end.notify();
           }
        });
        
        synchronized(end) {
            try {
                end.wait();
            } 
            catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        // --
        // Time to end playback
        // --
        
        try {
            trPlayService.stopRemotePlayback(securityKey.toString(), sessionKey);
        } 
        catch (RemoteException e) {
            e.printStackTrace();
        } 
        catch (SecurityException e) {
            e.printStackTrace();
        }
        
        mpr.stopProcessing();
    }

}
